/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Peers facility.
 *
 * This file contains the code to handle SCTP connections to a peer.
 *
 */

#include "waaad-internal.h"
#include "peer-internal.h"	

/*
 * FUNCTION:	_peer_sctp_create_server
 *
 * PARAMETERS:
 *  sock    : (out) On return, the identifier of the bound socket is stored here.
 *
 * DESCRIPTION: 
 *  Creates a listening SCTP bound socket.
 * It uses the data from configuration (local_addr_pri_sa, ...) to configure the server.
 *
 * RETURN VALUE:
 *   0 : The socket has been created, bound, and is listening for connections.
 *  !0 : an error occurred (standard errno are returned, such as ENOTSUP)
 */
int _peer_sctp_create_server( int * sock )
{
	int ret = 0;
	
	TRACE_ENTRY("%p", sock);
	
	TRACE_DEBUG(INFO, "Not implemented yet");
	return ENOTSUP;
}

/*
 * FUNCTION:	_peer_sctp_accept
 *
 * PARAMETERS:
 *  sock    : The listening server socket from which to accept clients.
 *  new_sd  : on return, the new socket for this client.
 *  ostreams: the number of outgoing streams.
 *  istreams: the number of incoming streams.
 *
 * DESCRIPTION: 
 *  Accepts an incoming connection and get the number of bi-lateral streams.
 *
 * RETURN VALUE:
 *   0 : A new client has arrived.
 *  !0 : an error occurred (standard errno are returned)
 */
int _peer_sctp_accept( int sock, int * new_sd, uint16_t *ostreams, uint16_t *istreams )
{
	int ret = 0;
	
	TRACE_ENTRY("%d %p %p %p", sock, new_sd, ostreams, istreams);
	
	TRACE_DEBUG(INFO, "Not implemented yet");
	return ENOTSUP;
}

/*
 * FUNCTION:	_peer_sctp_connect
 *
 * PARAMETERS:
 *  addr    : primary address of the server, that is passed to connect() function.
 *  addrlen : length of addr structure.
 *  sock    : (out) The client socket is stored here on return.
 *  ostreams: (in/out) the number of outbound streams (desired / negociated).
 *  istreams: (out) the number of inbound streams.
 *
 * DESCRIPTION: 
 *  Try and connect to a server and retrieve the number of streams in each direction.
 *
 * RETURN VALUE:
 *   0 : The client is connected.
 *  !0 : an error occurred (standard errno are returned)
 */
int _peer_sctp_connect( const sSA *addr, socklen_t addrlen, int *sock, uint16_t *ostreams, uint16_t *istreams )
{
	int ret = 0;
	
	TRACE_ENTRY("%p %d %p %p %p", addr, addrlen, sock, ostreams, istreams);
	
	TRACE_DEBUG(INFO, "Not implemented yet");
	return ENOTSUP;
}

/*
 * FUNCTION:	_peer_sctp_send
 *
 * PARAMETERS:
 *  socket  : The socket on which to send a message.
 *  streamid: The stream on which the message must be sent.
 *  data    : pointer to the data that must be sent.
 *  len     : length of the data buffer.
 *
 * DESCRIPTION: 
 *  Send some data over an SCTP association on a given stream.
 * The return values and error codes are the same as sendmsg().
 * If streamid is '-1' on function entry, a valid stream is picked
 * and set on function return. This can be use to automatically rotate
 * the streams on which messages are sent.
 *
 * RETURN VALUE:
 *   -1 : error (errno is set)
 *    0 : the socket has been close.
 *   >0 : number of bytes sent.
 */
ssize_t _peer_sctp_send(_peer_t * peer, int *streamid, const void * data, size_t len)
{
	int ret = 0;
	
	TRACE_ENTRY("%d %p(%d) %p %d", socket, streamid, streamid?*streamid:0, data, len);
	
	TRACE_DEBUG(INFO, "Not implemented yet");
	return ENOTSUP;
}

/*
 * FUNCTION:	_peer_sctp_recv
 *
 * PARAMETERS:
 *  peer    : The peer from which to receive the next message.
 *  streamid: the stream id on which data is received.
 *  data    : a buffer containing the received data.
 *  len     : the size of the data buffer
 *
 * DESCRIPTION: 
 *  Receive data and notifications on a SCTP socket.
 * When notifications or errors are received, the corresponding event is sent to the peer.
 * The data buffer is allocated and must be freed after use.
 * The return values and error codes are the same as recvmsg().
 *
 * RETURN VALUE:
 *  -1 : an error occurred
 *   0 : shutdown is in progress.
 *  >0 : length of data written in the data buffer ( also is *len ).
 */
ssize_t _peer_sctp_recv(_peer_t * peer, int * streamid, void ** data, size_t *len)
{
	int ret = 0;
	
	TRACE_ENTRY("%d %p %p %p", socket, streamid,  data, len);
	
	TRACE_DEBUG(INFO, "Not implemented yet");
	return ENOTSUP;
}

